/* DocPath.c -- Copyright 1989, 1993 Liam R. E. Quin.  All Rights Reserved.
 * This code is NOT in the public domain.
 * See the file COPYRIGHT for full details.
 *
 * $Id: DocPath.c,v 1.9 93/11/28 15:12:47 lee Exp $
 *
 */

#include "globals.h" /* defines and declarations for database filenames */

#include <sys/types.h>
#ifdef BSD
# include <strings.h>
# include <sys/param.h>
# define PATH_MAX MAXPATHLEN
#else /*not BSD*/
# include <string.h>
# include <limits.h> /* for PATH_MAX */
#endif

#include <stdio.h>

#include "emalloc.h"
#include "fileinfo.h"

/** Unix system calls: **/
extern int stat();

/** C Library functions: **/
extern int strlen();
extern char *strcpy();

/** extern lq-text functions **/
extern int IsDir(), IsFile(); /* from liblqutil */

/** Within this file: **/

/** **/

#ifdef ASCIITRACE
extern int AsciiTrace;
#endif

typedef struct s_DocPath {
    char *DirName;
    struct s_DocPath *Next;
} t_DocPath;

static t_DocPath *XDocPath = 0;

#ifndef PATH_MAX
# define PATH_MAX 2048
#endif

INTERNAL char *
_FindFile(Name)
    char *Name;
{
    t_DocPath *p;
    static char Buffer[PATH_MAX + 3]; /* +3 for ".Z\0" */

    if (!XDocPath) {
#ifdef ASCIITRACE
	if (AsciiTrace > 4) {
	    fprintf(stderr, "FindFile(%s) --> %s\n", Name,
					IsFile(Name) ? Name : (char *) 0);
	}
#endif
#ifdef COMPRESS_SUPPORT
	if (IsFile(Name)) return Name;
	(void) sprintf(Buffer, "%s.Z", Name);
	if (IsFile(Buffer)) return Buffer;
#endif /* COMPRESS_SUPPORT */
#ifdef GZIP_SUPPORT
	(void) sprintf(Buffer, "%s.gz", Name);
	if (IsFile(Buffer)) return Buffer;
#endif GZIP_SUPPORT
	return (char *) 0;
    }

    for (p = XDocPath; p; p = p->Next) {
	(void) sprintf(Buffer, "%s/%s", p->DirName, Name);
	if (IsFile(Buffer)) {
#ifdef ASCIITRACE
	    if (AsciiTrace > 4) {
		fprintf(stderr, "FindFile(%s) --> %s\n", Name, Buffer);
	    }
#endif
	    return Buffer;
	}
#ifdef COMPRESS_SUPPORT
	(void) strcat(Buffer, ".Z");
	if (IsFile(Buffer)) {
# ifdef ASCIITRACE
	    if (AsciiTrace > 4) {
		fprintf(stderr, "FindFile(%s) --> %s\n", Name, Buffer);
	    }
# endif
	    return Buffer;
	}
#endif /* COMPRESS_SUPPORT */
#ifdef GZIP_SUPPORT
	(void) sprintf(Buffer, "%s.gz", Name);
	if (IsFile(Buffer)) return Buffer;
#endif GZIP_SUPPORT
    }

    return (char*) 0;
}

API int
MakeDocPath(Path)
    char *Path;
{
    extern char *getenv();

    char *Start, *End;
    t_DocPath **dpp;

    if (XDocPath == (t_DocPath *) 0) {
	dpp = &XDocPath;
	*dpp = (t_DocPath *) 0;

	/* For each element in DocPath, */
	for (Start = Path; Start && *Start; Start = End) {
	    char SaveEnd;

	    /* find the end of this bit of the path */
	    for (End = Start; *End && *End != ':'; End++)
		;
	    
	    if (End == Start) break;

	    SaveEnd = (*End);
	    *End = '\0';

	    /* if not a directory, delete from path */
	    if (!IsDir(Start)) {
		*End = SaveEnd;
		continue;
	    }

	    /* add to the linked list */
	    *dpp = (t_DocPath *) emalloc("DocPath entry", sizeof(t_DocPath));
	    (*dpp)->DirName = emalloc("MakeDocPath.DirName",strlen(Start) + 1);
	    (void) strcpy((*dpp)->DirName, Start);
	    dpp = &(*dpp)->Next;
	    (*dpp) = (t_DocPath *) 0;
	    if ((*End = SaveEnd) != '\0') {
		End++;
	    }
	}
    }
    return 0;
}

